# Generated by roxygen2: do not edit by hand

S3method(safe_tensor_buffer,torch_tensor)
S3method(safe_tensor_meta,torch_tensor)
export(safe_load_file)
export(safe_save_file)
export(safe_serialize)
export(safe_tensor_buffer)
export(safe_tensor_meta)
export(safetensors)
export(safetensors_frameworks)
importFrom(R6,R6Class)
