

set -ex



pip check
cd bindings/python
pytest -v tests --ignore=tests/test_tf_comparison.py --ignore=tests/test_paddle_comparison.py --ignore=tests/test_flax_comparison.py --ignore=tests/test_mlx_comparison.py
exit 0
