



pip check
IF %ERRORLEVEL% NEQ 0 exit /B 1
cd bindings/python
IF %ERRORLEVEL% NEQ 0 exit /B 1
pytest -v tests --ignore=tests/test_tf_comparison.py --ignore=tests/test_paddle_comparison.py --ignore=tests/test_flax_comparison.py --ignore=tests/test_mlx_comparison.py
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
