# Torch API

[[autodoc]] safetensors.torch.load_file
[[autodoc]] safetensors.torch.load
[[autodoc]] safetensors.torch.save_file
[[autodoc]] safetensors.torch.save
[[autodoc]] safetensors.torch.load_model
[[autodoc]] safetensors.torch.save_model
