# metadata validations

    `metadata` must be a list.

---

    `metadata` must be a named list of scalar characters.

---

    `metadata` must be a named list of scalar characters.

---

    `metadata` must be a named list.

