% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\docType{data}
\name{safetensors_frameworks}
\alias{safetensors_frameworks}
\title{Reflection of supported frameworks}
\format{
An object of class \code{environment} of length 1.
}
\usage{
safetensors_frameworks
}
\description{
A reflection of supported frameworks.
}
\keyword{datasets}
