% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write.R
\name{safe_tensor_buffer}
\alias{safe_tensor_buffer}
\title{Get raw buffer from a tensor}
\usage{
safe_tensor_buffer(x)
}
\arguments{
\item{x}{(any)\cr
Tensor object.}
}
\value{
(\code{raw})
}
\description{
Convert a tensor object to a raw buffer in the formated expected by safetensors.
}
