% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write.R
\name{safe_tensor_meta}
\alias{safe_tensor_meta}
\title{Get metadata from a tensor}
\usage{
safe_tensor_meta(x)
}
\arguments{
\item{x}{(any)\cr
Tensor object.}
}
\value{
(\code{list})
}
\description{
Get the metadata from a tensor.
}
